#!/usr/bin/python

#	Copyright 2012 Jon Howell <jonh@jonh.net>
#
#    This file is part of shotgo.
#
#    shotgo is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    shotgo is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with shotgo.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import MySQLdb
import CoreDB
from PIL import Image

class Schema(CoreDB.Schema):
	def __init__(self, cxn, table_name, id_automatic=True):
		cur = cxn.cursor()
		result = cur.execute("select ordinal_position,column_name,data_type,is_nullable,column_default,column_key from information_schema.columns where table_name = %s", table_name)
		column_descriptors = cur.fetchall()
#		print ("\n".join(map(str, column_descriptors)))+"\n"
		CoreDB.Schema.__init__(self, table_name, column_descriptors, 1, id_automatic)

Row = CoreDB.Row

LEVEL_ADMIN = 8
LEVEL_ADMIN_FAMILY = 4
LEVEL_ADMIN_FAMILY_FRIENDS = 2
LEVEL_ADMIN_FAMILY_FRIENDS_CONTACTS = 1
LEVEL_EVERYBODY = 0

class PiwigoDB(CoreDB.DB):
	def __init__(self, host, user, password, db_name, photo_root):
		CoreDB.DB.__init__(self, db_name)
		self.cxn = MySQLdb.connect(host, user, password, db_name)
		self.short_cur = self.cursor()	# a cached read-only cursor for non-nested queries
		self.images_schema = Schema(self.cxn, "piwigo_images")
		self.categories_schema = Schema(self.cxn, "piwigo_categories")
		self.image_category_schema = Schema(self.cxn, "piwigo_image_category", False)
		self.tags_schema = Schema(self.cxn, "piwigo_tags")
		self.image_tag_schema = Schema(self.cxn, "piwigo_image_tag", False)
		self.photo_root = photo_root

	def cursor(self):
		return self.cxn.cursor()
	
	def fetchall(self, cur, result):
		return cur.fetchall()

	def sql_placeholder(self): return '%s'

	def clear(self):
		cur = self.cursor()
		for schema in (
			self.images_schema,
			self.categories_schema,
			self.image_category_schema):
			cur.execute("delete from %s;" % schema.table_name)

	def get_category(self, path):
		# (top) -> name='top' and id_uppercat=null -> 1
		# (top,middle) -> name='middle' and id_uppercat=1-> 2
		name = path[-1]
		upper_path = path[:-1]
		if (len(upper_path)==0):
			id_term = " and id_uppercat is null"
			values = (name,)
		else:
			id_term = " and id_uppercat=%s"
			id_uppercat = self.get_category(upper_path)
			values = (name,id_uppercat)

		existing_query = "select id from piwigo_categories where name=%s"+id_term
		cat_id = self.sql_get_unique_singleton(self.short_cur, existing_query, values)
#		print values
#		print "existing_query: ", existing_query
#		print "values: ", values
#		print existing_query % values

		if (cat_id==None):
			if (len(upper_path)==0):
				id_uppercat = None

#			uppercat_query = "select uppercats from piwigo_categories where id=%s"
#			parent_uppercats = self.sql_get_unique_singleton(
#				self.short_cur, uppercat_query, (id_uppercat,))
#			#this_uppercats = str(parent_uppercats+("+%d" % id_uppercat))
#			this_uppercats = "stub"

			new_cat_row = Row(self.categories_schema)
			new_cat_row["name"] = name
			new_cat_row["id_uppercat"] = id_uppercat
			new_cat_row["rank"] = 1
			new_cat_row["status"] = "public"
			new_cat_row["site_id"] = 1
			new_cat_row["visible"] = True
			new_cat_row["uppercats"] = "dummy"	# dummy value; gets replaced by piwigo
			new_cat_row["commentable"] = True
			new_cat_row["global_rank"] = 9

			self.insert_row(self.short_cur, new_cat_row)

			# re-run the existing_query to get the just-assigned cat_id
			cat_id = self.sql_get_unique_singleton(self.short_cur, existing_query, values)
#			print "existing_query %s" % existing_query
#			print "values %s" % str(values)
#			print "after insert, cat_id = %s" % cat_id

		return cat_id

	def md5sum(self, local_path):
		hdl = subprocess.Popen(["md5sum", local_path], stdout=subprocess.PIPE)
		value = hdl.stdout.read()
		hdl.wait()
		return value.split(" ")[0]

	def insert_photo(self, local_path, rel_path, md5sum, category_id, rotation, is_public):
		new_row = Row(self.images_schema)
		filename = os.path.basename(rel_path)
#		md5sum = self.md5sum(local_path)
		filesize_k = os.path.getsize(local_path)/1024

		# PIL is smart enough to not actually read the image's bits out,
		# just the header, if we only ever ask for the image size.
		imsize = Image.open(local_path).size

		if (is_public):
			level = LEVEL_EVERYBODY
		else:
			level = LEVEL_ADMIN_FAMILY_FRIENDS

		new_row["file"] = filename
		new_row["date_available"] = CoreDB.SQLNow()
		new_row["name"] = ".".join(filename.split(".")[:-1])
		new_row["hit"] = 0
		new_row["filesize"] = filesize_k
		new_row["width"] = imsize[0]
		new_row["height"] = imsize[1]
		new_row["path"] = self.photo_root + rel_path
		new_row["level"] = level
		new_row["md5sum"] = md5sum
		new_row["added_by"] = 1	# only one user in my world
		new_row["rotation"] = rotation

		self.insert_row(self.short_cur, new_row)
		image_id = self.get_image_id_by_md5sum(md5sum)

		new_img_cat = Row(self.image_category_schema)
#		print "image_id = %s" % image_id
#		print "category_id = %s" % category_id
		new_img_cat["image_id"] = image_id
		new_img_cat["category_id"] = category_id
		new_img_cat["rank"] = 3
		self.insert_row(self.short_cur, new_img_cat)
		return image_id

	def get_image_id_by_md5sum(self, md5sum):
		image_id = self.sql_get_unique_singleton(
			self.short_cur,
			"select id from %s where md5sum=%s" % (
				self.images_schema.table_name,
				self.sql_placeholder()),
			(md5sum,))
		return image_id

	def get_tag(self, tag_name):
		existing_query = "select id from %s where name=%s" % (
			self.tags_schema.table_name,
			self.sql_placeholder())
		values = (tag_name,)
		tag_id = self.sql_get_unique_singleton(
			self.short_cur, existing_query, values)
		if (tag_id == None):
			new_tag_row = Row(self.tags_schema)
			new_tag_row["name"] = tag_name
			new_tag_row["url_name"] = tag_name
			self.insert_row(self.short_cur, new_tag_row)
			# re-run the existing_query to get the just-assigned tag_id
			tag_id = self.sql_get_unique_singleton(
				self.short_cur, existing_query, values)
		return tag_id

	def assign_tag(self, md5sum, tag_name):
		new_row = Row(self.image_tag_schema)
		new_row["image_id"] = self.get_image_id_by_md5sum(md5sum)
		new_row["tag_id"] = self.get_tag(tag_name)
#		print "tag insert %s" % new_row.row_data
		self.insert_row(self.short_cur, new_row)
